;;  ------------------------------------------------------------------------
;; |                                                                        |
;; |   FILE           :  list-index.rkt                                     |
;; |   AUTHOR         :  Eugene Wallingford                                 |
;; |   CREATION DATE  :  2018/03/06                                         |
;; |                                                                        |
;; |   DESCRIPTION    :  local function, using letrec                       |
;; |                                                                        |
;;  ------------------------------------------------------------------------

#lang racket

;; -------------------------------------------------------------------------
;; list-index, with a stand-alone helper function
;; -------------------------------------------------------------------------

(define list-index
  (lambda (target los)
    (list-index-with-count target los 0)))

(define list-index-with-count
  (lambda (target los count)
    (cond ((null? los) -1)
          ((eq? target (first los)) count)
          (else (list-index-with-count target
                                       (rest los)
                                       (+ count 1)) ))))

;; -------------------------------------------------------------------------
;; list-index, with a local function using let
;; -------------------------------------------------------------------------

;(define list-index
;  (lambda (target los)
;    (let ((list-index-counted
;           (lambda (target los count)
;             (cond ((null? los) -1)
;                   ((eq? target (first los)) count)
;                   (else (list-index-counted target (rest los)
;                                                    (+ count 1)) )))))
;      (list-index-counted target los 0))))

;; -------------------------------------------------------------------------
;; list-index, with the local function "de-sugared"
;; -------------------------------------------------------------------------

;(define list-index
;  (lambda (target los)
;    ( (lambda (list-index-counted)
;        (list-index-counted target los 0))
;      (lambda (target los count)
;        (cond ((null? los) -1)
;              ((eq? target (first los)) count)
;              (else (list-index-counted target (rest los) (+ count 1)) ))))))

;; -------------------------------------------------------------------------
;; list-index, with a local function using letrec
;; -------------------------------------------------------------------------

;(define list-index
;  (lambda (target los)
;    (letrec ((list-index-counted
;              (lambda (target los base)
;                (cond ((null? los) -1)
;                      ((eq? target (first los)) base)
;                      (else (list-index-counted target
;                                                (rest los)
;                                                (+ base 1)) )))))
;      (list-index-counted target los 0))))

;; -------------------------------------------------------------------------
;; list-index, with a local function that reuses a bound parameter
;; -------------------------------------------------------------------------

;(define list-index
;  (lambda (target los)              ;; once target is passed in ...
;    (letrec ((list-index-counted
;              (lambda (los base)
;                (cond ((null? los) -1)                   ;; ... its value
;                      ((eq? target (first los)) base)    ;; never changes
;                      (else (list-index-counted (rest los) (+ base 1)) )))))
;      (list-index-counted los 0))))

;; -------------------------------------------------------------------------

